@@ -1,9 +1,13 @@
-#Flab
-#DeviceManager
-#Version 2.0.0
-#Published 17-Jul-2022
-#Distributed under GNU GPL v3
-#Author: Nicholas Jose
+# Flab
+# DeviceManager
+# Version 2.0.2
+# Published XX-XXX-XXXX
+# Distributed under GNU GPL v3
+# Author: Nicholas Jose
+
+"""
+The DeviceManager module contains classes and methods for creating and manipulating devices
+"""
 
 import importlib
 import os
@@ -11,25 +15,36 @@ import glob
 import types
 from multiprocessing.managers import NamespaceProxy, SyncManager
 
-#A class for sharing device objects across flab/multiprocessing
+
 class FlabDeviceManager(SyncManager):
+    """A manager class for sharing devices in a flab object and across threads and processes.
+    Extends the multiprocessing class SyncManager
+    """
     pass
 
-#A class for dynamically creating device objects
-#Device objects in 1.0.0 have publicly accessible attributes
-class DeviceManager():
 
-    description = 'Actions for loading and reloading device libraries into flab'
-    version = '2.0.0'
-    devices = {}
-    flab_device_manager = FlabDeviceManager()
-    load_all_devices_completed = False
+class DeviceManager:
+    """
+    A class for dynamically creating and sharing device objects. Devices do not necessarily have publicly accessible
+    attributes, and a "get_attr" method may be needed to access these attributes.
+    Version 2.0.2
+    """
+
+    version = '2.0.2'
 
     def __init__(self):
-        pass
+        self.devices = {}
+        self.flab_device_manager = FlabDeviceManager()
+        self.load_all_devices_completed = False
 
-    # register a device within a project with the FlabDeviceManager
     def register_device(self, device_name):
+        """
+        Registers a device within a project with the FlabDeviceManager
+
+        :param device_name: name of the device
+        :type device_name: str
+        :returns: None
+        """
         try:
             cwf = 'Projects.' + os.path.split(os.getcwd())[1]
             device_module = importlib.import_module(cwf + '.Devices.' + device_name)
@@ -45,21 +60,36 @@ class DeviceManager():
 
     # register a list of devices within a project with the FlabDeviceManager
     def register_devices(self, device_names):
+        """registers a list of devices within a project with the FlabDeviceManager. This is required
+        before the devices are loaded into a flab object.
+
+        :param device_names: array of device names
+        :type device_names: [str]
+
+        :return: None
+        """
         for device_name in device_names:
             self.register_device(device_name)
 
-    # register all devices within a project with the FlabDeviceManager
     def register_all_devices(self):
+        """
+        Registers all devices within a project with the FlabDeviceManager
+
+        :returns: None
+        """
         cwd = os.getcwd()
-        devices = glob.glob(cwd+'/Devices/*.py')
+        devices = glob.glob(cwd + '/Devices/*.py')
         device_names = []
         for d in devices:
-            device_names.append(d[len(cwd+'/Devices/'):].replace('.py',''))
+            device_names.append(d[len(cwd + '/Devices/'):].replace('.py', ''))
         self.register_devices(sorted(device_names))
 
-
-    #star the flab device manager
     def start_device_manager(self):
+        """
+        starts the device manager
+
+        :returns: None
+        """
         try:
             self.flab_device_manager.start()
         except Exception as e:
@@ -67,13 +97,20 @@ class DeviceManager():
         finally:
             pass
 
-    #Dynamically load a predefined set of devices into flab as objects. The device is defined in the Devices folder
     def load_device(self, device_name):
+        """
+        Loads a predefined set of devices into a flab object as objects. The device is defined in the Devices folder.
+
+        :param device_name: name of the device
+        :type device_name: str
+
+        :returns: None
+        """
         if device_name not in self.modules:
             self.display(device_name + ' is not registered')
         else:
             try:
-                create_method = getattr(self.flab_device_manager,device_name)
+                create_method = getattr(self.flab_device_manager, device_name)
                 new_device = create_method()
                 new_device.set_flab(self)
                 self.devices.update({device_name: new_device})
@@ -85,6 +122,13 @@ class DeviceManager():
                 pass
 
     def load_devices(self, device_names):
+        """
+        Loads a list of devices into a flab object
+
+        :param device_names: [str]
+
+        :returns: None
+        """
         try:
             for d in device_names:
                 self.load_device(d)
@@ -95,12 +139,17 @@ class DeviceManager():
             pass
 
     def load_all_devices(self):
+        """
+        Loads all devices in a project folder into a flab object
+
+        :returns: None
+        """
         try:
             cwd = os.getcwd()
-            devices = glob.glob(cwd+'/Devices/*.py')
+            devices = glob.glob(cwd + '/Devices/*.py')
             device_names = []
             for d in devices:
-                device_names.append(d[len(cwd+'/Devices/'):].replace('.py',''))
+                device_names.append(d[len(cwd + '/Devices/'):].replace('.py', ''))
             self.load_devices(sorted(device_names))
         except Exception as e:
             self.display('Error in loading all devices')
@@ -108,9 +157,11 @@ class DeviceManager():
         finally:
             self.load_all_devices_completed = True
 
-    #Dynaically reload a single device into flab - this only reloads the main device class, not the assoicated drivers and protocols
-    #This is now deprecated as of v1.0.0
     def reload_device(self, device_name):
+        """
+        Reloads a single device into flab. This only reloads the main device class, not the associated drivers and
+        protocols. This is now deprecated after v1.0.0
+        """
         try:
             device_module = importlib.reload(self.modules[device_name])
             new_device = self.modules[device_name].Device()
@@ -125,6 +176,11 @@ class DeviceManager():
             pass
 
     def get_devices(self):
+        """
+        returns devices in a flab object
+
+        :returns: a list of device names
+        """
         devices = {}
         keys = self.devices.keys()
         try:
@@ -136,8 +192,15 @@ class DeviceManager():
             self.display(e)
         return devices
 
-#A proxy class for sharing devices
+
 def Proxy(target):
+    """
+    A method for creating proxy classes for devices.
+
+    :param target: the target device object
+
+    :returns: DeviceProxy
+    """
 
     class DeviceProxy(NamespaceProxy):
         _exposed_ = tuple(dir(target))
@@ -147,10 +210,8 @@ def Proxy(target):
             if isinstance(result, types.MethodType):
                 def wrapper(*args, **kwargs):
                     return self._callmethod(name, args)  # Note the return here
+
                 return wrapper
             return result
 
     return DeviceProxy
-
-
-
