@@ -1,7 +1,7 @@
 #Flab
 #DriverTemplate.py
 #A template class for device drivers
-#Version 1.1.2
+#Version 2.0.1
 #Published 15-Jul-2022
 #Distributed under GNU GPL v3
 #Author: Nicholas Jose
@@ -9,7 +9,7 @@
 class Driver():
 
     driver_name = 'DriverTemplate'
-    version = '2.0.0'
+    version = '2.0.1'
 
     def __init__(self):
         pass
@@ -27,5 +27,5 @@ class Driver():
         return self.driver_name
 
     #sets the name of a Driver
-    def set_driver_name(self, s):
-        self.driver_name = s
+    def set_driver_name(self, driver_name):
+        self.driver_name = driver_name
