@@ -1,7 +1,7 @@
 #Flab
 #ProtocolTemplate.py
 #A template class for device protocols
-#Version 1.1.2
+#Version 2.0.1
 #Published 15-Jul-2022
 #Distributed under GNU GPL v3
 #Author: Nicholas Jose
@@ -9,7 +9,7 @@
 class Protocol():
 
     protocol_name = 'ProtocolTemplate'
-    version = '2.0.0'
+    version = '2.0.1'
 
     def __init__(self):
         pass
@@ -27,7 +27,7 @@ class Protocol():
         return self.protocol_name
 
     #sets the name of a Protocol
-    def set_protocol_name(self, s):
-        self.protocol_name = s
+    def set_protocol_name(self, protocol_name):
+        self.protocol_name = protocol_name
 
 
